<?php

namespace Bixo\Organization\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Bixo\Organization\Forms\Organization as OrganizationForm;
use Bixo\Organization\Http\Requests\OrganizationResourceRequest;
use Bixo\Organization\Http\Resources\OrganizationResource;
use Bixo\Organization\Http\Resources\OrganizationsCollection;
use Bixo\Organization\Models\Organization;
use Bixo\Organization\Actions\OrganizationAction;
use Bixo\Organization\Actions\OrganizationActions;


/**
 * Resource controller class for organization.
 */
class OrganizationResourceController extends BaseController
{

    /**
     * Initialize organization resource controller.
     *
     *
     * @return null
     */
    public function __construct()
    {
        parent::__construct();
        $this->middleware(function ($request, $next) {
            $this->form = OrganizationForm::grouped(false)
                            ->setAttributes()
                            ->toArray();
            $this->modules = $this->modules(config('bixo.organization.modules'), 'organization', guard_url('organization'));
            return $next($request);
        });
    }

    /**
     * Display a list of organization.
     *
     * @return Response
     */
    public function index(OrganizationResourceRequest $request)
    {
        $request = $request->all();
        $page = OrganizationActions::run('paginate', $request);

        $data = new OrganizationsCollection($page);

        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('organization::organization.names'))
            ->view('organization::organization.index')
            ->data(compact('data', 'modules', 'form'))
            ->output();

    }

    /**
     * Display organization.
     *
     * @param Request $request
     * @param Model   $organization
     *
     * @return Response
     */
    public function show(OrganizationResourceRequest $request, Organization $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new OrganizationResource($model);
        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('organization::organization.name'))
            ->data(compact('data', 'form', 'modules'))
            ->view('organization::organization.show')
            ->output();
    }

    /**
     * Show the form for creating a new organization.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(OrganizationResourceRequest $request, Organization $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new OrganizationResource($model);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('organization::organization.name'))
            ->view('organization::organization.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Create new organization.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(OrganizationResourceRequest $request, Organization $model)
    {
        try {
            $request = $request->all();
            $model = OrganizationAction::run('store', $model, $request);
            $data = new OrganizationResource($model);
            return $this->response->message(trans('messages.success.created', ['Module' => trans('organization::organization.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('organization/organization/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/organization/organization'))
                ->redirect();
        }

    }

    /**
     * Show organization for editing.
     *
     * @param Request $request
     * @param Model   $organization
     *
     * @return Response
     */
    public function edit(OrganizationResourceRequest $request, Organization $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new OrganizationResource($model);
        // return view('organization::organization.edit', compact('data', 'form', 'modules'));

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('organization::organization.name'))
            ->view('organization::organization.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Update the organization.
     *
     * @param Request $request
     * @param Model   $organization
     *
     * @return Response
     */
    public function update(OrganizationResourceRequest $request, Organization $model)
    {
        try {
            $request = $request->all();
            $model = OrganizationAction::run('update', $model, $request);
            $data = new OrganizationResource($model);

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('organization::organization.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('organization/organization/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('organization/organization/' .  $model->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the organization.
     *
     * @param Model   $organization
     *
     * @return Response
     */
    public function destroy(OrganizationResourceRequest $request, Organization $model)
    {
        try {

            $request = $request->all();
            $model = OrganizationAction::run('destroy', $model, $request);
            $data = new OrganizationResource($model);

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('organization::organization.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('organization/organization/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('organization/organization/' .  $model->getRouteKey()))
                ->redirect();
        }

    }
}
