<?php

namespace Bixo\Organization\Http\Controllers;

use Exception;
use Bixo\Organization\Actions\OrganizationWorkflow;
use Bixo\Organization\Http\Requests\OrganizationWorkflowRequest;
use Bixo\Organization\Http\Resources\OrganizationResource;
use Bixo\Organization\Models\Organization;
use Litepie\Http\Controllers\ActionController as BaseController;

/**
 * Workflow controller class.
 *
 */
class OrganizationWorkflowController extends BaseController
{
    /**
     * Action controller function for organization.
     *
     * @param Model $organization
     * @param action next action for the organization.
     *
     * @return Response
     */
    public function __invoke(OrganizationWorkflowRequest $request, Organization $organization, $trasition)
    {
        try {
            $request = $request->all();
            $organization = OrganizationWorkflow::run($trasition, $organization, $request);
            $data = new OrganizationResource($organization);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('organization::organization.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('organization/organization/' . $organization->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('organization/organization/' . $organization->getRouteKey()))
                ->redirect();
        }
    }


    /**
     * Get the organization and user for the workflow.
     *
     * @param PublicRequest $request
     * @param string $organization
     * @param string $user
     *
     * @return Response
     */
    public function get(PublicRequest $request, $organization, $user)
    {
        if (!$request->hasValidSignature()) {
            abort(403, 'This url is expired!');
        }

        $auth = Auth::guard()->getProvider()->getModel();
        $user = $auth::findBySignedId($user);
        Auth::guard()->login($user);
        $organization = Organization::findBySignedId($organization);
        $transitions = $this->transitions($organization, $user);
        $user = user();
        Auth::logout();
        return $this->response->setMetaTitle(trans('organization::organization.name') . ' ' . trans('Approvals'))
            ->view('organization::public.organization.workflow')
            ->layout('mobile')
            ->data(compact('organization', 'user', 'transitions'))
            ->output();
    }

    /**
     * Display the workflow form for the organization.
     *
     * @param PublicRequest $request
     * @param string $organization
     * @param string $user
     *
     * @return Response
     */
    public function post(PublicRequest $request, $organization, $user)
    {
        if (!$request->hasValidSignature()) {
            abort(403, 'This url is expired!');
        }
        $transition = $request->transition;
        $auth = Auth::guard()->getProvider()->getModel();
        $user = $auth::findBySignedId($user);
        Auth::guard()->login($user);
        $organization = Organization::findBySignedId($organization);
        $user = user();
        $organization = OrganizationWorkflow::run($transition, $organization, $request->all());
        Auth::logout();

        return response()->json(
            [
                'status' => 'success',
                'url' => $request->url(),
            ]
        );
    }

    private function transitions($organization, $user)
    {
        $transitions = [];
        foreach ($organization->workflow()->transitions($organization) as $key => $value) {
            $name = $value->getName();
            $array['url'] = URL::temporarySignedRoute('litecms.organization.workflow', now()->addMinutes(3), [
                'guard' => 'admin',
                'transition' => $name,
                'organization' => $organization->getSignedId(now()->addMinutes(3)),
                'user' => $user->getSignedId(now()->addMinutes(3)),
                'trans' => 'en',
            ]
            );
            $array['name'] = $name;
            $array['key'] = $name;
            $array['form'] = $organization->workflow()->form($value);
            $array['label'] = trans('organization::organization.workflow.' . $name);
            $transitions[] = $array;
        }
        return $transitions;
    }
}
